import json
import logging
import aiohttp
from typing import List
from mcp import types
from protein_data_bank_mcp.rest_api.constants import PDB_API_URL

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


async def fetch_data(api_suffix: str) -> List[types.TextContent]:
    logger.info(f"Fetching data from {api_suffix}")
    url = f"{PDB_API_URL}{api_suffix}"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            response_json = await response.json()
    result = types.TextContent(
        type="text", text=json.dumps(response_json, indent=2))
    return result



from typing import List
from mcp import types
from protein_data_bank_mcp.rest_api.utils import fetch_data


async def chemical_component(comp_id: str) -> List[types.TextContent]:
    api_suffix = f"/core/chemcomp/{comp_id}"
    result = await fetch_data(api_suffix)
    return result


async def drugbank_annotations(comp_id: str) -> List[types.TextContent]:
    api_suffix = f"/core/drugbank/{comp_id}"
    result = await fetch_data(api_suffix)
    return result



from typing import List
from mcp import types
from protein_data_bank_mcp.rest_api.utils import fetch_data


async def assembly_schema() -> List[types.TextContent]:
    api_suffix = "/schema/assembly"
    return await fetch_data(api_suffix)


async def branched_entity_instance_schema() -> List[types.TextContent]:
    api_suffix = "/schema/branched_entity_instance"
    return await fetch_data(api_suffix)


async def branched_entity_schema() -> List[types.TextContent]:
    api_suffix = "/schema/branched_entity"
    return await fetch_data(api_suffix)


async def chem_comp_schema() -> List[types.TextContent]:
    api_suffix = "/schema/chem_comp"
    return await fetch_data(api_suffix)


async def drugbank_schema() -> List[types.TextContent]:
    api_suffix = "/schema/drugbank"
    return await fetch_data(api_suffix)


async def entry_schema() -> List[types.TextContent]:
    api_suffix = "/schema/entry"
    return await fetch_data(api_suffix)


async def nonpolymer_entity_instance_schema() -> List[types.TextContent]:
    api_suffix = "/schema/nonpolymer_entity_instance"
    return await fetch_data(api_suffix)


async def nonpolymer_entity_schema() -> List[types.TextContent]:
    api_suffix = "/schema/nonpolymer_entity"
    return await fetch_data(api_suffix)


async def polymer_entity_instance_schema() -> List[types.TextContent]:
    api_suffix = "/schema/polymer_entity_instance"
    return await fetch_data(api_suffix)


async def polymer_entity_schema() -> List[types.TextContent]:
    api_suffix = "/schema/polymer_entity"
    return await fetch_data(api_suffix)


async def pubmed_schema() -> List[types.TextContent]:
    api_suffix = "/schema/pubmed"
    return await fetch_data(api_suffix)


async def uniprot_schema() -> List[types.TextContent]:
    api_suffix = "/schema/uniprot"
    return await fetch_data(api_suffix)



import json
import logging

from mcp import types
from typing import Dict
from Bio.PDB import PDBList, PDBParser
from pathlib import Path


logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)


PROTEIN_COMMON_ONE_TO_THREE = {
    "A": "ALA",
    "R": "ARG",
    "N": "ASN",
    "D": "ASP",
    "C": "CYS",
    "Q": "GLN",
    "E": "GLU",
    "G": "GLY",
    "H": "HIS",
    "I": "ILE",
    "L": "LEU",
    "K": "LYS",
    "M": "MET",
    "F": "PHE",
    "P": "PRO",
    "S": "SER",
    "T": "THR",
    "W": "TRP",
    "Y": "TYR",
    "V": "VAL",
}

PROTEIN_COMMON_THREE_TO_ONE = {v: k for k, v in PROTEIN_COMMON_ONE_TO_THREE.items()}


class PDBStore:
    def __init__(self, folder: Path | str):
        self.folder = Path(folder)
        self.folder.mkdir(parents=True, exist_ok=True)
        self._pdb_list = PDBList()
        self._parser = PDBParser()

    def download_pdb(self, entry_id: str):
        self._pdb_list.retrieve_pdb_file(
            entry_id, pdir=self.folder, overwrite=True, file_format="pdb"
        )

    def get_pdb(self, entry_id: str):
        pdb_path = self.folder / f"pdb{entry_id.lower()}.ent"
        if not pdb_path.exists():
            self.download_pdb(entry_id)
        return pdb_path

    def get_residue_chains(self, entry_id: str) -> Dict[str, str]:
        path = self.get_pdb(entry_id)
        structure = self._parser.get_structure(entry_id, path)

        chains = {}
        for chain in structure.get_chains():
            residues = "".join(
                [
                    PROTEIN_COMMON_THREE_TO_ONE.get(x.resname, "X")
                    for x in chain.get_residues()
                ]
            )
            chains[chain.id] = residues

        return chains


parser = PDBStore("temp")


def get_residue_chains(entry_id: str) -> types.TextContent:
    global parser
    return json.dumps(parser.get_residue_chains(entry_id), indent=2)



from typing import List
from mcp import types
from protein_data_bank_mcp.rest_api.utils import fetch_data


async def structure(entry_id: str) -> List[types.TextContent]:
    api_suffix = f"/core/entry/{entry_id}"
    result = await fetch_data(api_suffix)
    return result


async def pubmed_annotations(entry_id: str) -> List[types.TextContent]:
    api_suffix = f"/core/pubmed/{entry_id}"
    result = await fetch_data(api_suffix)
    return result



from typing import List
from mcp import types
from protein_data_bank_mcp.rest_api.utils import fetch_data


async def pdb_cluster_data_aggregation(group_id: str) -> List[types.TextContent]:
    api_suffix = f"/core/entry_groups/{group_id}"
    result = await fetch_data(api_suffix)
    return result


async def aggregation_group_provenance(
    group_provenance_id: str,
) -> List[types.TextContent]:
    api_suffix = f"/core/group_provenance/{group_provenance_id}"
    result = await fetch_data(api_suffix)
    return result


async def pdb_cluster_data_aggregation_method(group_id: str) -> List[types.TextContent]:
    api_suffix = f"/core/polymer_entity_groups/{group_id}"
    result = await fetch_data(api_suffix)
    return result



from typing import List
from mcp import types
from protein_data_bank_mcp.rest_api.utils import fetch_data


async def branched_entity_instance(
    entry_id: str, asym_id: str
) -> List[types.TextContent]:
    api_suffix = f"/core/branched_entity_instance/{entry_id}/{asym_id}"
    result = await fetch_data(api_suffix)
    return result


async def non_polymer_entity_instance(
    entry_id: str, asym_id: str
) -> List[types.TextContent]:
    api_suffix = f"/core/non_polymer_entity_instance/{entry_id}/{asym_id}"
    result = await fetch_data(api_suffix)
    return result


async def polymer_entity_instance(
    entry_id: str, asym_id: str
) -> List[types.TextContent]:
    api_suffix = f"/core/polymer_entity_instance/{entry_id}/{asym_id}"
    result = await fetch_data(api_suffix)
    return result



from typing import List
from mcp import types
from protein_data_bank_mcp.rest_api.utils import fetch_data


async def current_entry_ids() -> List[types.TextContent]:
    api_suffix = "/holdings/current/entry_ids"
    result = await fetch_data(api_suffix)
    return result


async def structure_status(entry_id: str) -> List[types.TextContent]:
    api_suffix = f"/holdings/status/{entry_id}"
    result = await fetch_data(api_suffix)
    return result


async def structure_list_status(entry_ids: List[str]) -> List[types.TextContent]:
    api_suffix = f"/holdings/status?entry_ids={','.join(entry_ids)}"
    result = await fetch_data(api_suffix)
    return result


async def removed_structure_description(entry_id: str) -> List[types.TextContent]:
    api_suffix = f"/holdings/removed/{entry_id}"
    result = await fetch_data(api_suffix)
    return result


async def removed_entry_ids() -> List[types.TextContent]:
    api_suffix = "/holdings/removed/entry_ids"
    result = await fetch_data(api_suffix)
    return result


async def unreleased_structures(ids: List[str]) -> List[types.TextContent]:
    api_suffix = f"/holdings/unreleased?ids={','.join(ids)}"
    result = await fetch_data(api_suffix)
    return result


async def unreleased_structure_processing(entry_id: str) -> List[types.TextContent]:
    api_suffix = f"/holdings/unreleased/{entry_id}"
    result = await fetch_data(api_suffix)
    return result


async def unreleased_entry_ids() -> List[types.TextContent]:
    api_suffix = "/holdings/unreleased/entry_ids"
    result = await fetch_data(api_suffix)
    return result



import os
import logging

from dotenv import load_dotenv, find_dotenv
from mcp.server import FastMCP
from protein_data_bank_mcp.rest_api.assembly import structural_assembly_description
from protein_data_bank_mcp.rest_api.chemical_component import (
    chemical_component,
    drugbank_annotations,
)
from protein_data_bank_mcp.rest_api.entity_instance import (
    polymer_entity_instance,
    branched_entity_instance,
    non_polymer_entity_instance,
)
from protein_data_bank_mcp.rest_api.entity import (
    branched_entity,
    polymer_entity,
    non_polymer_entity,
    uniprot_annotations,
)
from protein_data_bank_mcp.rest_api.entry import structure, pubmed_annotations
from protein_data_bank_mcp.rest_api.groups import (
    aggregation_group_provenance,
    pdb_cluster_data_aggregation,
    pdb_cluster_data_aggregation_method,
)
from protein_data_bank_mcp.rest_api.interface import pairwise_polymeric_interface_description
from protein_data_bank_mcp.pdb_store.storage import get_residue_chains

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)

load_dotenv(find_dotenv())

mcp_server = FastMCP("protein_data_bank_mcp")

tools = [
    structural_assembly_description,
    chemical_component,
    drugbank_annotations,
    polymer_entity_instance,
    branched_entity_instance,
    non_polymer_entity_instance,
    uniprot_annotations,
    branched_entity,
    polymer_entity,
    non_polymer_entity,
    structure,
    pubmed_annotations,
    aggregation_group_provenance,
    pdb_cluster_data_aggregation,
    pdb_cluster_data_aggregation_method,
    pairwise_polymeric_interface_description,
    get_residue_chains,
]

for tool in tools:
    mcp_server.tool()(tool)

def main():
    mcp_server.run(transport="stdio")


from typing import List
from mcp import types
from protein_data_bank_mcp.rest_api.utils import fetch_data


async def pairwise_polymeric_interface_description(
    entry_id: str, assembly_id: str, interface_id: str
) -> List[types.TextContent]:
    api_suffix = f"/core/interface/{entry_id}/{assembly_id}/{interface_id}"
    result = await fetch_data(api_suffix)
    return result



import pytest

from src.protein_data_bank_mcp.pdb_store.storage import PDBStore


@pytest.fixture
def pdb_store(tmp_path):
    return PDBStore(tmp_path)


def test_download_pdb(pdb_store: PDBStore):
    entry_id = "1fat"
    path = pdb_store.get_pdb(entry_id)
    assert path.is_file()


def test_get_residues(pdb_store: PDBStore):
    entry_id = "1fat"
    residues = pdb_store.get_residue_chains(entry_id)
    assert len(residues) == 4
